@@ -1,13 +1,5 @@
 context("MIP: model")
 
-test_that("one can set an objective function", {
-  m <- MIPModel()
-  m <- add_variable(m, x[i], i = 1:10, type = "binary")
-  m <- set_objective(m, x[1] + x[3], sense = "min")
-  expect_equal(m$objective$sense, "min")
-  expect_equal(deparse(m$objective$expression[[1]]), "x[1] + x[3]")
-})
-
 test_that("only max and min are valid directions for an objective function", {
   m <- add_variable(MIPModel(), x[i], i = 1:10, type = "binary")
   expect_error(set_objective(m, x[1] + x[3], sense = "wat"))
@@ -20,13 +12,6 @@ test_that("all symbols in an obj. function need to be variables", {
   expect_error(set_objective(m, x[5], sense = "min"))
 })
 
-test_that("obj. function can bind external variables", {
-  w <- c(1, 2)
-  m <- add_variable(MIPModel(), x[i], i = 1:2, type = "binary")
-  m <- set_objective(m, w[1] * x[1], sense = "min")
-  expect_equal(deparse(m$objective$expression[[1]]), "1 * x[1]")
-})
-
 test_that("set_objective throws an error if it is non-linear", {
   m <- add_variable(MIPModel(), x[i], i = 1:3, type = "binary")
   expect_error(set_objective(m, sum_expr(x[i], i = 1:2) * x[3]))
@@ -66,11 +51,12 @@ test_that("we can model a tsp", {
     r <- MIPModel() %>%
       add_variable(x[i, j], i = 1:cities, j = 1:cities, type = "binary") %>%
       set_objective(sum_expr(distance_matrix[i, j] * x[i, j],
-                            i = 1:cities, j = 1:cities), sense = "min") %>%
+        i = 1:cities, j = 1:cities
+      ), sense = "min") %>%
       add_constraint(x[i, i] == 0, i = 1:cities) %>%
       add_constraint(x[i, j] == x[j, i], i = 1:cities, j = 1:cities) %>%
       add_constraint(sum_expr(x[i, j], i = sub_tours[[s]], j = sub_tours[[s]]) <=
-                       length(sub_tours[s]) - 1, s = 1:length(sub_tours))
+        length(sub_tours[s]) - 1, s = 1:length(sub_tours))
   )
 })
 
@@ -101,8 +87,8 @@ test_that("bug 20161011 #83: bounds of binary vars are not 0/1", {
   model <- add_variable(MIPModel(), x, type = "binary") %>%
     add_constraint(x <= 10) %>%
     set_objective(-x, sense = "max")
-  expect_equal(0, model$variables[[1]]$lb)
-  expect_equal(1, model$variables[[1]]$ub)
+  expect_equal(model$variable_bounds_lower, 0)
+  expect_equal(model$variable_bounds_upper, 1)
 })
 
 test_that("multiplications in objective fun", {
@@ -110,7 +96,9 @@ test_that("multiplications in objective fun", {
     add_variable(y, type = "continuous", ub = 4) %>%
     add_constraint(x + y <= 10) %>%
     set_objective(5 * (-x + y), sense = "max")
-  expect_equal(deparse(m$objective$expression[[1]]), "-5 * x + 5 * y")
+  expect_equal(m$objective$fun@constant, 0)
+  expect_equal(m$objective$fun@terms[[1]]@coefficient, -5)
+  expect_equal(m$objective$fun@terms[[2]]@coefficient, 5)
 })
 
 test_that("model output works without an obj. function", {
@@ -118,16 +106,7 @@ test_that("model output works without an obj. function", {
   expect_output(show(m))
 })
 
-test_that("devision in objective fun", {
-  m <- add_variable(MIPModel(), x, type = "continuous", lb = 4) %>%
-    add_variable(y, type = "continuous", ub = 4) %>%
-    add_constraint(x + y <= 10) %>%
-    set_objective(5 / (-x + y), sense = "max")
-  expect_equal(deparse(m$objective$expression[[1]]), "-0.2 * x + 0.2 * y")
-})
-
 test_that("small to mid sized models should work", {
-  skip_on_cran()
   n <- 400
   expect_silent(result <- MIPModel() %>%
     add_variable(x[i], i = 1:n, type = "binary") %>%
@@ -141,30 +120,12 @@ test_that("bug 20160713 #41: quantifiers in constraints in sum_expr", {
     add_constraint(sum_expr(x[i], i = 1:3 + y) == 1, y = c(0, 3, 6)))
 })
 
-test_that("small to mid sized model should work #2", {
-  skip_on_cran()
-  n <- 40
-  coef <- matrix(1:(n ^ 2), ncol = 2)
-  expect_silent(MIPModel() %>%
-    add_variable(x[i, j], i = 1:n, j = 1:n) %>%
-    add_constraint(sum_expr(coef[i, j] * x[i, j], i = 1:n, j = 1:n) == 1))
-})
-
 test_that("bug 20160729: two sum_expr on one side", {
-  m <- MIPModel() %>%
-    add_variable(x[j], j = 1:4) %>%
-    add_constraint(sum_expr(x[j], j = 1:2) - sum_expr(x[j], j = 3:4) == 0)
-  expect_equal(deparse(m$constraints[[1]]$lhs[[1]]),
-               "x[1L] + x[2L] + (-1 * x[3L] + -1 * x[4L])")
-})
-
-test_that("bug 20160729_2: external var binding", {
-  x <- 1:10
-  m <- MIPModel() %>%
-    add_variable(x[j], j = 1:2)
-  expect_warning(add_constraint(m, sum_expr(x[j], j = 1:2) == 0))
-  expect_warning(add_constraint(m, sum_expr(x[j], j = 1:2) == 0),
-                 regexp = "x")
+  expect_silent({
+    m <- MIPModel() %>%
+      add_variable(x[j], j = 1:4) %>%
+      add_constraint(sum_expr(x[j], j = 1:2) - sum_expr(x[j], j = 3:4) == 0)
+  })
 })
 
 test_that("solve_model warns about wrong arguments", {
@@ -180,48 +141,81 @@ test_that("set_objective_ supports standard eval.", {
 
 test_that("can expand a term N * (x - y)", {
   m <- add_variable(MIPModel(), x[i], i = 1:2)
-  m <- set_objective_(m, ~ -5 * (x[1] - x[2]))
-  expect_equal("-5 * x[1] + 5 * x[2]",
-               deparse(m$objective$expression[[1]]))
+  expect_silent(
+    set_objective_(m, ~ -5 * (x[1] - x[2]))
+  )
 })
 
 test_that("evaluates terms", {
   m <- add_variable(MIPModel(), x[i], i = 1:2)
   m <- set_objective_(m, ~ 5 * 5)
-  expect_equal("25",
-               deparse(m$objective$expression[[1]]))
+  expect_equal(25, m$objective$fun)
 })
 
 test_that("SE handles sum_expr well", {
-  m <- MIPModel() %>%
-    add_variable_(~x[j], j = 1:4) %>%
-    add_constraint_(~sum_expr(x[j], j = 1:2, j == 1) -
-                      sum_expr(x[j], j = 3:4) == 0)
-  expect_equal(deparse(m$constraints[[1]]$lhs[[1]]),
-               "x[1L] + (-1 * x[3L] + -1 * x[4L])")
-})
-
-test_that("bug 20161108 #105: c can be assigned as var name", {
-  expect_warning(add_variable(MIPModel(), c, lb = 1, ub = 2) %>%
-    set_objective(c) %>%
-    add_constraint(c <= 5) %>%
-    set_bounds(c, lb = 1.5), regexp = "interfere",  all = TRUE)
+  expect_silent({
+    MIPModel() %>%
+      add_variable_(~ x[j], j = 1:4) %>%
+      add_constraint_(~ sum_expr(x[j], j = 1:2, j == 1) -
+        sum_expr(x[j], j = 3:4) == 0)
+  })
 })
 
 test_that("bug 20161110 #106: Error when indices used in sum_expr(...)
            condition already have values in workspace", {
-   i <- 2
-   j <- 2
-   model <- MIPModel()
-   model <- add_variable(model, x[i, j], i = 1:2, j = 1:2, i != j)
-   expect_silent(result <- set_objective(model,
-                                         sum_expr(x[i, j], i = 1:2,
-                                                  j = 1:2, i != j)))
-   expect_silent(result <- add_constraint(model,
-                                         sum_expr(x[i, j], i = 1:2,
-                                                  j = 1:2, i != j) <= 10))
-   expect_silent(result <- add_constraint(model,
-                                          sum_expr(1 + x[i, j] + x[i, j],
-                                                   i = 1:2, j = 1:2,
-                                                   i != j) <= 10))
+  i <- 2
+  j <- 2
+  model <- MIPModel()
+  model <- add_variable(model, x[i, j], i = 1:2, j = 1:2, i != j)
+  expect_silent(result <- set_objective(
+    model,
+    sum_expr(x[i, j],
+      i = 1:2,
+      j = 1:2, i != j
+    )
+  ))
+  expect_silent(result <- add_constraint(
+    model,
+    sum_expr(x[i, j],
+      i = 1:2,
+      j = 1:2, i != j
+    ) <= 10
+  ))
+  expect_silent(result <- add_constraint(
+    model,
+    sum_expr(1 + x[i, j] + x[i, j],
+      i = 1:2, j = 1:2,
+      i != j
+    ) <= 10
+  ))
+})
+
+test_that("MIPModel supports a numeric objective", {
+  model <- MIPModel()
+  model <- add_variable(model, x[i, j], i = 1:2, j = 1:2, i != j)
+  model <- set_objective(model, 42)
+  res <- objective_function(model)
+  expect_equal(res$constant, 42)
+})
+
+test_that("MIPModel edge case work", {
+  model <- MIPModel()
+  model <- add_variable(model, x[i, j], i = 1:2, j = 1:2, i != j)
+  model <- set_objective(model, x[1, 2] + x[1, 2] + x[1, 2] + x[1, 2])
+  res <- objective_function(model)
+  expect_equal(res$constant, 0)
+  expect_equal(as.numeric(res$solution), c(4, 0))
+})
+
+test_that("MIPModel add_variable signals some errors", {
+  model <- MIPModel()
+  model <- add_variable(model, x[i, j], i = 1:2, j = 1:2, i != j)
+  expect_error(
+    add_variable(model, x[i, j], i = 1:2, j = 1:2, i != j),
+    "already"
+  )
+  expect_error(
+    add_variable(model, sum(x), i = 1:2, j = 1:2, i != j),
+    "form"
+  )
 })
